﻿using ProjectMonorail.Scripts.Exceptions;
using ProjectMonorail.Scripts.Monorail.Drawnings;
using System.Diagnostics;
using System.Text;

namespace ProjectMonorail.Scripts.Monorail.CollectionGenericObjects
{
    /// <summary>
    /// Класс-хранилище коллекций
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class StorageCollection<T>
    where T : DrawingMonorail
    {
        /// <summary>
        /// Словарь (хранилище) с коллекциями
        /// </summary>
        readonly Dictionary<CollectionInfo, ICollectionGenericObjects<T>> _storages;

        /// <summary>
        /// Возвращение списка названий коллекций
        /// </summary>
        public List<CollectionInfo> Keys => _storages.Keys.ToList();

        /// <summary>
        /// Ключевое слово, с которого должен начинаться файл
        /// </summary>
        private readonly string _collectionKey = "CollectionsStorage";

        /// <summary>
        /// Разделитель для записи ключа и значения элемента словаря
        /// </summary>
        private readonly string _separatorForKeyValue = "|";

        /// <summary>
        /// Разделитель для записей коллекции данных в файл
        /// </summary>
        private readonly string _separatorItems = ";";

        /// <summary>
        /// Конструктор
        /// </summary>
        public StorageCollection()
        {
            _storages = new Dictionary<CollectionInfo, ICollectionGenericObjects<T>>();
        }

        /// <summary>
        /// Добавление коллекции в хранилище
        /// </summary>
        /// <param name="name">Название коллекции</param>
        /// <param name="collectionType">тип коллекции</param>
        public void AddCollection(string name, CollectionType collectionType)
        {
            // TODO проверка, что name не пустой и нет в словаре записи с таким ключом
            // TODO Прописать логику для добавления
            CollectionInfo collectionInfo = new CollectionInfo(name, collectionType, string.Empty);

            if (_storages.ContainsKey(collectionInfo)) return;

            if (collectionType == CollectionType.None) return;
            else if (collectionType == CollectionType.Massive) _storages[collectionInfo] = new MassiveGenericObjects<T>();
            else if (collectionType == CollectionType.List) _storages[collectionInfo] = new ListGenericObjects<T>();
        }

        /// <summary>
        /// Удаление коллекции
        /// </summary>
        /// <param name="name">Название коллекции</param>
        public void DelCollection(string name)
        {
            // TODO Прописать логику для удаления коллекции

            CollectionInfo collectionInfo = new CollectionInfo(name, CollectionType.None, string.Empty);
            if (_storages.ContainsKey(collectionInfo))
                _storages.Remove(collectionInfo);
        }

        /// <summary>
        /// Доступ к коллекции
        /// </summary>
        /// <param name="name">Название коллекции</param>
        /// <returns></returns>
        public ICollectionGenericObjects<T>? this[string name]
        {
            get
            {
                // TODO Продумать логику получения 

                CollectionInfo collectionInfo = new CollectionInfo(name, CollectionType.None, string.Empty);
                if (_storages.ContainsKey(collectionInfo))
                    return _storages[collectionInfo];
                return null;
            }
        }

        /// <summary>
        /// Сохранение информации по автомобилям в хранилище в файл
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - сохранение прошло успешно, false - ошибка при сохранении данных</returns>
        public void SaveData(string filename)
        {
            if (_storages.Count == 0)
            {
                throw new Exception("Storage is empty");
            }
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }
            using (StreamWriter writer = new StreamWriter(filename))
            {
                writer.Write(_collectionKey);
                foreach (KeyValuePair<CollectionInfo, ICollectionGenericObjects<T>> value in _storages)
                {
                    StringBuilder stringBuilder = new StringBuilder();
                    stringBuilder.Append(Environment.NewLine);

                    if (value.Value.Count == 0)
                    {
                        continue;
                    }

                    stringBuilder.Append(value.Key);
                    stringBuilder.Append(_separatorForKeyValue);
                    stringBuilder.Append(value.Value.MaxCount);
                    stringBuilder.Append(_separatorForKeyValue);

                    foreach (T? item in value.Value.GetItems())
                    {
                        string data = item?.GetDataForSave() ?? string.Empty;
                        if (string.IsNullOrEmpty(data))
                        {
                            continue;
                        }
                        stringBuilder.Append(data);
                        stringBuilder.Append(_separatorItems);
                    }
                    writer.Write(stringBuilder);
                }
            }
        }

        /// <summary>
        /// Загрузка информации по автомобилям в хранилище из файла
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - загрузка прошла успешно, false - ошибка при загрузке данных</returns>
        public void LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                throw new Exceptions.FileDoesNotExistException("Файл не существует " + filename);
            }

            using (StreamReader streamRader = File.OpenText(filename))
            {
                string inputString = streamRader.ReadLine();

                if (inputString == null || inputString.Length == 0)
                {
                    throw new Exceptions.FileEmptyException("Файл пустой " + filename);
                }

                if (!inputString.StartsWith(_collectionKey))
                {
                    throw new Exceptions.ObjectNotFoundException();
                }

                _storages.Clear();
                string strs = "";
                while ((strs = streamRader.ReadLine()) != null)
                {
                    string[] record = strs.Split(_separatorForKeyValue, StringSplitOptions.RemoveEmptyEntries);
                    if (record.Length != 3)
                    {
                        continue;
                    }

                    CollectionInfo? collectionInfo = CollectionInfo.GetCollectionInfo(record[0]) ?? throw new Exception("Не удалось определить информацию коллекции: " + record[0]);
                    ICollectionGenericObjects<T>? collection = StorageCollection<T>.CreateCollection(collectionInfo.CollectionType) ?? throw new Exception("Не удалось создать коллекцию");

                    if (collection == null)
                    {
                        throw new Exceptions.ObjectNotFoundException();
                    }
                    collection.MaxCount = Convert.ToInt32(record[1]);
                    string[] set = record[2].Split(_separatorItems, StringSplitOptions.RemoveEmptyEntries);

                    foreach (string elem in set)
                    {
                        var obj = elem?.CreateDrawingMonorail();
                        if (obj is T monorail)
                        {
                            if (collection.Insert(monorail) == -1)
                            {
                                throw new Exception("Объект не удалось добавить в коллекию: " + record[3]);
                            }
                        }
                    }
                    
                    _storages.Add(collectionInfo, collection);
                }
            }
        }

        private static ICollectionGenericObjects<T>? CreateCollection(CollectionType collectionType)
        {
            return collectionType switch
            {
                CollectionType.Massive => new MassiveGenericObjects<T>(),
                CollectionType.List => new ListGenericObjects<T>(),
                _ => null,
            };
        }

        public T? this[int index]
        {
            get
            {
                if (index >= 0 && index < Keys.Count && Keys[index] != null)
                {
                    return (T)_storages[Keys[index]];
                }
                return null;
            }
        }
    }
}