﻿using ProjectMonorail.Scripts.Monorail.Entities;
using System.Diagnostics;
using System.Drawing.Drawing2D;

namespace ProjectMonorail.Scripts.Monorail.Drawnings
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение базового объекта-сущности
    /// </summary>
    public class DrawingMonorail
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityMonorail? EntityMonorail { get; protected set; }

        /// <summary>
        /// Ширина окна
        /// </summary>
        private int? _pictureWidth;

        /// <summary>
        /// Высота окна
        /// </summary>
        private int? _pictureHeight;

        /// <summary>
        /// Левая координата прорисовки монорельса
        /// </summary>
        protected int? _startPositionX;

        /// <summary>
        /// Верхняя кооридната прорисовки монорельса
        /// </summary>
        protected int? _startPositionY;

        /// <summary>
        /// Ширина прорисовки монорельса
        /// </summary>
        private readonly int _drawningMonorailWidth = 90;

        /// <summary>
        /// Высота прорисовки монорельса
        /// </summary>
        private readonly int _drawningMonorailHeight = 40;

        /// <summary>
        /// Координаты X объекта
        /// </summary>
        public int? GetPositionX => _startPositionX;

        /// <summary>
        /// Координаты Y объекта
        /// </summary>
        public int? GetPositionY => _startPositionY;

        /// <summary>
        /// Ширина объекта
        /// </summary>
        public int GetWidth => _drawningMonorailWidth;

        /// <summary>
        /// Высота объекта
        /// </summary>
        public int GetHeight => _drawningMonorailHeight;

        /// <summary>
        /// Пустой конструкотор
        /// </summary>
        private DrawingMonorail() 
        {
            _pictureWidth = null;
            _pictureHeight = null;
            _startPositionX = null;
            _startPositionY = null;
        }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Основной цвет</param>
        public DrawingMonorail(int speed, double weight, Color bodyColor) : this()
        {
            EntityMonorail = new EntityMonorail(speed, weight, bodyColor);
        }

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="entityMonorail"></param>
        public DrawingMonorail(EntityMonorail entityMonorail) : this()
        {
            Debug.WriteLine(entityMonorail.BodyColor.ToString() + " " + entityMonorail.Speed + " " + entityMonorail.Weight);
            EntityMonorail = new EntityMonorail(entityMonorail.Speed, entityMonorail.Weight, entityMonorail.BodyColor);
        }

        /// <summary>
        /// Конструктор для наследников
        /// </summary>
        /// <param name="drawningMonorailWidth">Ширина прорисовки монорельса</param>
        /// <param name="drawningMonorailHeight">Высота прорисовки монорельса</param>
        protected DrawingMonorail(int drawningMonorailWidth, int drawningMonorailHeight) : this()
        {
            _drawningMonorailWidth = drawningMonorailWidth;
            _drawningMonorailHeight = drawningMonorailHeight;
        }

        /// <summary>
        /// Установка границ поля
        /// </summary>
        /// <param name="width">Ширина поля</param>
        /// <param name="height">Высота поля</param>
        /// <returns> true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
        public bool SetPictureSize(int width, int height)
        {
            // TODO проверка, что объект "влезает" в размеры поля
            // если влезает, сохраняем границы и корректируем позицию объекта, если она была уже установлена
            if (_drawningMonorailWidth > width || _drawningMonorailHeight > height) return false;

            if (_startPositionX.HasValue && _startPositionY.HasValue)
            {
                if (_startPositionX.Value + _drawningMonorailWidth > width)
                {
                    _startPositionX = width - _drawningMonorailWidth;
                }
                if (_startPositionY.Value + _drawningMonorailHeight > height)
                {
                    _startPositionY = height - _drawningMonorailHeight;
                }
            }

            _pictureWidth = width;
            _pictureHeight = height;
            return true;
        }

        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
        {
            if (!_pictureHeight.HasValue || !_pictureWidth.HasValue) return;

            // TODO если при установке объекта в эти координаты, он будет "выходить" за границы формы
            // то надо изменить координаты, чтобы он оставался в этих границах

            if (x + _drawningMonorailWidth > _pictureWidth.Value || x < 0)
            {
                Random random = new();
                _startPositionX = random.Next(0, _pictureWidth.Value - _drawningMonorailWidth);
            }
            else
            {
                _startPositionX = x;
            }

            if (y + _drawningMonorailHeight > _pictureHeight.Value || y < 0)
            {
                Random random = new();
                _startPositionY = random.Next(0, _pictureHeight.Value - _drawningMonorailHeight);
            }
            else
            {
                _startPositionY = y;
            }
        }

        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        /// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
        public bool MoveTransport(DirectionType direction)
        {
            if (EntityMonorail == null || !_startPositionX.HasValue || !_startPositionY.HasValue) return false;

            switch (direction)
            {
                case DirectionType.Left:
                    if (_startPositionX.Value - EntityMonorail.Step > 0)
                    {
                        _startPositionX -= (int)EntityMonorail.Step;
                    }
                    return true;
                case DirectionType.Up:
                    if (_startPositionY.Value - EntityMonorail.Step > 0)
                    {
                        _startPositionY -= (int)EntityMonorail.Step;
                    }
                    return true;
                case DirectionType.Right:
                    if (_startPositionX.Value + EntityMonorail.Step + _drawningMonorailWidth < _pictureWidth)
                    {
                        _startPositionX += (int)EntityMonorail.Step;
                    }
                    return true;
                case DirectionType.Down:
                    if (_startPositionY.Value + EntityMonorail.Step + _drawningMonorailHeight < _pictureHeight)
                    {
                        _startPositionY += (int)EntityMonorail.Step;
                    }
                    return true;
                default:
                    return false;
            }
        }

        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        public virtual void DrawTransport(Graphics g)
        {
            if (EntityMonorail == null || !_startPositionX.HasValue || !_startPositionY.HasValue) return;

            Pen pen_Body = new Pen(EntityMonorail.BodyColor, 1.5f);

            Brush brush_Body = new SolidBrush(EntityMonorail.BodyColor);

            Debug.WriteLine("+");
            Debug.WriteLine(EntityMonorail.BodyColor.Name);

            //Кузов монорельса
            GraphicsPath pointsMonorailBody_LeftBody = new GraphicsPath();
            pointsMonorailBody_LeftBody.AddLine(_startPositionX.Value + 8, _startPositionY.Value + 10, _startPositionX.Value + 13, _startPositionY.Value);
            pointsMonorailBody_LeftBody.AddLine(_startPositionX.Value + 13, _startPositionY.Value, _startPositionX.Value + 80, _startPositionY.Value);
            pointsMonorailBody_LeftBody.AddLine(_startPositionX.Value + 80, _startPositionY.Value, _startPositionX.Value + 80, _startPositionY.Value + 25);
            pointsMonorailBody_LeftBody.AddLine(_startPositionX.Value + 80, _startPositionY.Value + 25, _startPositionX.Value + 8, _startPositionY.Value + 25);
            pointsMonorailBody_LeftBody.AddLine(_startPositionX.Value + 8, _startPositionY.Value + 25, _startPositionX.Value + 8, _startPositionY.Value + 10);
            g.DrawLine(pen_Body, _startPositionX.Value + 8, _startPositionY.Value + 10, _startPositionX.Value + 30, _startPositionY.Value + 10);
            g.DrawLine(pen_Body, _startPositionX.Value + 39, _startPositionY.Value + 10, _startPositionX.Value + 80, _startPositionY.Value + 10);
            g.DrawPath(pen_Body, pointsMonorailBody_LeftBody);

            //Дверь
            g.DrawRectangle(pen_Body, _startPositionX.Value + 30, _startPositionY.Value + 5, 9, 15);

            //Окна
            g.DrawRectangle(pen_Body, _startPositionX.Value + 14, _startPositionY.Value + 2, 5, 6);
            g.DrawRectangle(pen_Body, _startPositionX.Value + 21, _startPositionY.Value + 2, 5, 6);
            g.DrawRectangle(pen_Body, _startPositionX.Value + 70, _startPositionY.Value + 2, 5, 6);

            //Связка монорельса
            g.FillRectangle(brush_Body, _startPositionX.Value + 80, _startPositionY.Value + 2, 7, 22);

            //Нижняя часть монорельса
            PointF[] pointsLowerPartMonorail_LeftSide = new PointF[8];
            pointsLowerPartMonorail_LeftSide[0] = new PointF(_startPositionX.Value, _startPositionY.Value + 30);
            pointsLowerPartMonorail_LeftSide[1] = new PointF(_startPositionX.Value + 8, _startPositionY.Value + 25);
            pointsLowerPartMonorail_LeftSide[2] = new PointF(_startPositionX.Value + 72, _startPositionY.Value + 25);
            pointsLowerPartMonorail_LeftSide[3] = new PointF(_startPositionX.Value + 36, _startPositionY.Value + 25);
            pointsLowerPartMonorail_LeftSide[4] = new PointF(_startPositionX.Value + 36, _startPositionY.Value + 32);
            pointsLowerPartMonorail_LeftSide[5] = new PointF(_startPositionX.Value + 15, _startPositionY.Value + 32);
            pointsLowerPartMonorail_LeftSide[6] = new PointF(_startPositionX.Value + 15, _startPositionY.Value + 35);
            pointsLowerPartMonorail_LeftSide[7] = new PointF(_startPositionX.Value, _startPositionY.Value + 32);
            g.FillPolygon(brush_Body, pointsLowerPartMonorail_LeftSide);

            PointF[] pointsLowerPartMonorail_RightSide = new PointF[7];
            pointsLowerPartMonorail_RightSide[0] = new PointF(_startPositionX.Value + 86, _startPositionY.Value + 30);
            pointsLowerPartMonorail_RightSide[1] = new PointF(_startPositionX.Value + 80, _startPositionY.Value + 25);
            pointsLowerPartMonorail_RightSide[2] = new PointF(_startPositionX.Value + 50, _startPositionY.Value + 25);
            pointsLowerPartMonorail_RightSide[3] = new PointF(_startPositionX.Value + 50, _startPositionY.Value + 32);
            pointsLowerPartMonorail_RightSide[4] = new PointF(_startPositionX.Value + 69, _startPositionY.Value + 32);
            pointsLowerPartMonorail_RightSide[5] = new PointF(_startPositionX.Value + 79, _startPositionY.Value + 35);
            pointsLowerPartMonorail_RightSide[6] = new PointF(_startPositionX.Value + 86, _startPositionY.Value + 32);
            g.FillPolygon(brush_Body, pointsLowerPartMonorail_RightSide);
            g.FillRectangle(brush_Body, _startPositionX.Value + 8, _startPositionY.Value + 25, 70, 3);

            //Колеса
            Brush brush_White = new SolidBrush(Color.White);
            g.FillEllipse(brush_White, _startPositionX.Value + 10, _startPositionY.Value + 30, 10, 10);
            g.DrawEllipse(pen_Body, _startPositionX.Value + 10, _startPositionY.Value + 30, 10, 10);

            g.FillEllipse(brush_White, _startPositionX.Value + 25, _startPositionY.Value + 30, 10, 10);
            g.DrawEllipse(pen_Body, _startPositionX.Value + 25, _startPositionY.Value + 30, 10, 10);

            g.FillEllipse(brush_White, _startPositionX.Value + 65, _startPositionY.Value + 30, 10, 10);
            g.DrawEllipse(pen_Body, _startPositionX.Value + 65, _startPositionY.Value + 30, 10, 10);

            g.FillEllipse(brush_White, _startPositionX.Value + 50, _startPositionY.Value + 30, 10, 10);
            g.DrawEllipse(pen_Body, _startPositionX.Value + 50, _startPositionY.Value + 30, 10, 10);
        }
    }
}
